﻿using System;
using System.Windows.Input;
using System.IO.Ports;
using System.ComponentModel;
using System.Linq;

namespace AvlF8101.F8101
{
    public class Avl : INotifyPropertyChanged
    {
        public delegate void InvokeScriptHandler(string scriptName, params object[] args);

        public delegate void InvokeMessageBoxHandler(string message);

        public Avl(InvokeScriptHandler invokeScript, InvokeMessageBoxHandler invokeMessageBox)
        {
            this._invokeScript = invokeScript;
            this._invokeMessageBox = invokeMessageBox;
            this._rxLog = new RxLog();
            this._rxLog.TextReceived += appendLog;
            this._rxLog.PositionReceived += updateMarker;
            _rs232c.DataReceived += new Rs232c.Rs232c.DataReceivedHandler(this._rxLog.AnalizeReceiveData);
            this.ClearLogCommand = new DelegateCommand(() => { this.Log = ""; });
            this.SelectChannelCommand = new DelegateCommand(this.selectChannel);
            this.SetOnPriorityNetworkCommand = new DelegateCommand(this.setOnPriorityNetwork);
            this.SetOffPriorityNetworkCommand = new DelegateCommand(this.setOffPriorityNetwork);
            this.StopScanCommand = new DelegateCommand(this.stopScan);
            this.StartScanCommand = new DelegateCommand(this.startScan);
            this.HungUpCommand = new DelegateCommand(this.hungUp);
            this.SelcallCommand = new DelegateCommand(this.selcall);
            this.AleCommand = new DelegateCommand(this.ale);
        }

        private Rs232c.Rs232c _rs232c = new Rs232c.Rs232c()
        {
            PortName = "",
            BaudRate = 38400,
            DataBits = 8,
            Parity = Parity.None,
            StopBits = StopBits.One,
        };

        private RxLog _rxLog;

        private InvokeScriptHandler _invokeScript;

        private void updateMarker(string id, double latitude, double longitude)
        {
            this._invokeScript("moveMarker", id, dmmToDd(latitude), dmmToDd(longitude));
        }

        private double dmmToDd(double position)
        {
            return (int)(position / 100) + ((position % 100) / 60);
        }

        private void appendLog(string value)
        {
            this.Log += value;
        }

        private string _log;

        public string Log
        {
            get
            {
                return this._log;
            }
            set
            {
                this._log = value;
                this.NotifyPropertyChanged("Log");
            }
        }

        private string _channel = "";

        public string Channel
        {
            get
            {
                return this._channel;
            }
            set
            {
                this._channel = value;
                this.NotifyPropertyChanged("Channel");
            }
        }

        public ICommand ClearLogCommand { get; private set; }

        private void selectChannel()
        {
            byte[] command = new byte[] { 0xFE, 0xFE, 0x8A, 0xDF, 0x1A, 0x38 };
            rs232cWriteData(
                command.Concat(System.Text.Encoding.ASCII.GetBytes(_channel))
                .Concat(new byte[] { 0xFD })
                .ToArray());
        }

        public ICommand SelectChannelCommand { get; set; }

        private string _priorityNetwork = "";

        public string PriorityNetwork
        {
            get
            {
                return this._priorityNetwork;
            }
            set
            {
                this._priorityNetwork = value;
                this.NotifyPropertyChanged("PriorityNetwork");
            }
        }

        private void setOnPriorityNetwork()
        {
            setPriorityNetwork(true);
        }

        public ICommand SetOnPriorityNetworkCommand { get; set; }

        private void setOffPriorityNetwork()
        {
            setPriorityNetwork(false);
        }

        public ICommand SetOffPriorityNetworkCommand { get; set; }

        private void setPriorityNetwork(bool value)
        {
            byte[] command = new byte[] { 0xFE, 0xFE, 0x8A, 0xDF, 0x1A, 0x50 };
            byte [] data = value ? new byte[] { 0x01 } : new byte[] { 0x00 };
            rs232cWriteData(
                command.Concat(data)
                .Concat(System.Text.Encoding.ASCII.GetBytes(_priorityNetwork))
                .Concat(new byte[] { 0xFD })
                .ToArray());
        }

        private void stopScan()
        {
            rs232cWriteData(new byte[] { 0xFE, 0xFE, 0x8A, 0xDF, 0x0E, 0x00, 0xFD });
        }

        public ICommand StopScanCommand { get; set; }

        private void startScan()
        {
            rs232cWriteData(new byte[] { 0xFE, 0xFE, 0x8A, 0xDF, 0x0E, 0x01, 0xFD });
        }

        public ICommand HungUpCommand { get; set; }

        private void hungUp()
        {
            rs232cWriteData(new byte[] { 0xFE, 0xFE, 0x8A, 0xDF, 0x1A, 0x40, 0xFD });
        }

        public ICommand StartScanCommand { get; set; }

        private SelcallCallType _selcallCallType;

        public SelcallCallType SelcallCallType
        {
            get
            {
                return _selcallCallType;
            }
            set
            {
                _selcallCallType = value;
                this.NotifyPropertyChanged("SelcallCallType");
            }
        }

        private string _selcallId = "";

        public string SelcallId
        {
            get
            {
                return _selcallId;
            }
            set
            {
                _selcallId = value;
                this.NotifyPropertyChanged("SelcallId");
            }
        }

        private string _phone = "";

        public string Phone
        {
            get
            {
                return _phone;
            }
            set
            {
                _phone = value;
                this.NotifyPropertyChanged("Phone");
            }
        }

        private string _pin = "";

        public string Pin
        {
            get
            {
                return _pin;
            }
            set
            {
                _pin = value;
                this.NotifyPropertyChanged("Pin");
            }
        }

        private string _selcallMessage = "";

        public string SelcallMessage
        {
            get
            {
                return _selcallMessage;
            }
            set
            {
                _selcallMessage = value;
                this.NotifyPropertyChanged("SelcallMessage");
            }
        }

        private void selcall()
        {
            if (_selcallCallType == null)
            {
                return;
            }
            byte[] command = new byte[] { 0xFE, 0xFE, 0x8A, 0xDF, 0x1A, 0x42 };
            rs232cWriteData(
                command.Concat(_selcallCallType.MakeCommand(_selcallId, _phone, _pin, _selcallMessage))
                .Concat(new byte[] { 0xFD })
                .ToArray());
        }

        public ICommand SelcallCommand { get; set; }
        
        private AleCallType _aleCallType;

        public AleCallType AleCallType
        {
            get
            {
                return _aleCallType;
            }
            set
            {
                _aleCallType = value;
                this.NotifyPropertyChanged("AleCallType");
            }
        }

        private string _aleId = "";

        public string AleId
        {
            get
            {
                return _aleId;
            }
            set
            {
                _aleId = value;
                this.NotifyPropertyChanged("AleId");
            }
        }

        private string _aleMessage = "";

        public string AleMessage
        {
            get
            {
                return _aleMessage;
            }
            set
            {
                _aleMessage = value;
                this.NotifyPropertyChanged("AleMessage");
            }
        }

        private void ale()
        {
            if (_aleCallType == null)
            {
                return;
            }
            byte[] command = new byte[] { 0xFE, 0xFE, 0x8A, 0xDF, 0x1A, 0x41 };
            rs232cWriteData(
                command.Concat(_aleCallType.MakeCommand(_aleId, _aleMessage))
                .Concat(new byte[] { 0xFD })
                .ToArray());
        }

        public ICommand AleCommand { get; set; }

        private InvokeMessageBoxHandler _invokeMessageBox;

        private void rs232cWriteData(byte[] data)
        {
            try
            {
                this._rs232c.WriteData(data);
            }
            catch (Exception ex)
            {
                this._invokeMessageBox(ex.Message);
            }
        }

        private bool _isOpen = false;

        public bool IsOpen
        {
            get
            {
                return _isOpen;
            }
        }

        public void Open()
        {
            try
            {
                this._rs232c.Open();
            }
            catch (Exception ex)
            {
                this._invokeMessageBox(ex.Message);
            }
            this._isOpen = true;
            this.NotifyPropertyChanged("IsOpen");
        }

        public void Close()
        {
            try
            {
                this._rs232c.Close();
            }
            catch (Exception ex)
            {
                this._invokeMessageBox(ex.Message);
            }
            this._isOpen = false;
            this.NotifyPropertyChanged("IsOpen");
        }
        
        public string Port
        {
            set
            {
                this._rs232c.PortName = value;
            }
            get
            {
                return _rs232c.PortName;
            }
        }

        public int BaudRate
        {
            set
            {
                this._rs232c.BaudRate = value;
            }
            get
            {
                return _rs232c.BaudRate;
            }
        }

        public int DataBits
        {
            set
            {
                this._rs232c.DataBits = value;
            }
            get
            {
                return _rs232c.DataBits;
            }
        }

        public Parity Parity
        {
            set
            {
                this._rs232c.Parity = value;
            }
            get
            {
                return _rs232c.Parity;
            }
        }

        public StopBits StopBits
        {
            set
            {
                this._rs232c.StopBits = value;
            }
            get
            {
                return _rs232c.StopBits;
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void NotifyPropertyChanged(params string[] names)
        {
            if (this.PropertyChanged != null)
            {
                foreach (string name in names)
                {
                    this.PropertyChanged(this, new PropertyChangedEventArgs(name));
                }
            }
        }
    }
}
